GameManager = class()

function GameManager:created(x)
    -- you can accept and set parameters here
    self.gameOver = false
    self.flashOpacity = 0
    self.flashSpeed = 0.33
    self.xPos = 0
    
    self.restart = gesture.tap(function(gest)
        self.beginning = true
        self.gameOver = false
        self.flashOpacity = 0
        self.xPos = 0
        self:reset()
        self.scene.ball.body2d.linearVelocity = vec2(0)
        self.scene.score1.score = 0
        self.scene.score2.score = 0
    end, 2,3)
    
    self.close = gesture.tap(function(gest)
        viewer.close()
    end, 3,2)
    
    self.beginning = true 
end

function GameManager:fixedUpdate()
    if not self.gameOver then
        local halfWidth = (WIDTH / HEIGHT) * (self.scene.camera:get(camera).orthoSize / 2)
        self:fixVelocity()
        
        local ball = self.scene.ball
        if ball.worldPosition.x + (ballDiameter /2) >= halfWidth then
            ball.body2d.linearVelocity = vec2(0)
            ball.x = halfWidth - (ballDiameter /2)  - 0.01
            self.xPos = -(halfWidth / 2)
            self.gameOver = true
            score1.score = score1.score  + 1
        elseif ball.worldPosition.x - (ballDiameter /2) <= -halfWidth then
            ball.body2d.linearVelocity = vec2(0)
            ball.x = -halfWidth + (ballDiameter /2)  + 0.01
            self.xPos = halfWidth / 2
            self.gameOver = true
            score2.score = score2.score  + 1
        end 
        
        if self.gameOver then
            tween(self):to{flashOpacity = 110}:time(self.flashSpeed):to{flashOpacity = 0}:time(self.flashSpeed):loop(3):onComplete(function()
                self:reset()
            end)
            sound.play(asset.Score_Sound)
        end
    end
end

function GameManager:fixVelocity()
    local ball = self.scene.ball
    local vel = ball.body2d.linearVelocity
    if vel.x > 0 then
        vel = (vel/vel.x) * ballSpeed
    end
    ball.body2d.linearVelocity = vel
end

function GameManager:reset()
    self.scene.ball.worldPosition = vec3(0)
    self.scene.paddle1.y = 0
    self.scene.paddle2.y = 0
    if not self.beginning then 
        tween({}):to{}:time(0.5):onComplete(function()
            self:startBall()
            self.gameOver = false
        end)
    end
end
i = 0
function GameManager:draw()
    if self.gameOver then
        local halfHeight = (self.scene.camera:get(camera).orthoSize / 2)
        local halfWidth = (WIDTH/HEIGHT) * halfHeight
        
        style.push().fill(186, 110, 110, self.flashOpacity).sortOrder(2).shapeMode(CENTER)
            rect(self.xPos,0, halfWidth, 2 * halfHeight)
        style.pop()
    end
    
    if self.beginning then
        style.push().fill(79, 49, 49).sortOrder(-3).shapeMode(CENTER)
        rect(0,0, 16, 2.4)
        style.pop()
        if self.beginning then
            style.push().fill(255).sortOrder(-3).shapeMode(CENTER).textAlign(CENTER | MIDDLE).fontSize(1).font("Rockwell")
            text("Touch To Start",0,-0.2)
            style.pop()
        end
        
        matrix.push().transform2d(8,-0.2, 0.45,0.45, -90)
        style.push().fill(186, 110, 110, 205).sortOrder(-3).shapeMode(CENTER).textAlign(CENTER | MIDDLE).fontSize(1).font("Rockwell")
        text("Touch and drag to move the paddle.\nTriple tap twice to restart.\nDouble tap three times to close game.",0,0)
        style.pop()
        matrix.pop()
        
        matrix.push().transform2d(-8,-0.2, 0.45,0.45, 90)
        style.push().fill(186, 110, 110, 205).sortOrder(-3).shapeMode(CENTER).textAlign(CENTER | MIDDLE).fontSize(1).font("Rockwell")
        text("Touch and drag to move the paddle.\nTriple tap twice to restart.\nDouble tap three times to close game.",0,0)
        style.pop()
        matrix.pop()
    end


    
end

function GameManager:startBall()
    if not self.beginning then
        local vel = vec2(1,0.25)
        if math.random(1,2) % 2 == 1 then
            vel.x = vel.x * -1
        end
        if math.random(1,2) % 2 == 1 then
            vel.y = vel.y * -1
        end
        
        self.scene.ball.body2d.linearVelocity = ballSpeed * vel
    end
end

function GameManager:touched(touch)
    if self.beginning then
        self.beginning = false
        self:startBall()
    end
    return false
end
